@@ -274,11 +274,11 @@ void ComputeThread::cell_finished_running(DataCell::id_t id)
 void ComputeThread::on_message(websocketpp::connection_hdl hdl, message_ptr msg)
 	{
 	client::connection_ptr con = wsclient.get_con_from_hdl(hdl);
-	// std::cerr << msg->get_payload() << std::endl;
+	// std::cerr << "payload: " << msg->get_payload() << std::endl;
 
 	// Parse the JSON message.
 	nlohmann::json root;
-	
+
 	try {
 		root=nlohmann::json::parse(msg->get_payload());
 		}
@@ -311,17 +311,24 @@ void ComputeThread::on_message(websocketpp::connection_hdl hdl, message_ptr msg)
 	else
 		cell_id.created_by_client=false;
 	// std::cerr << "received cell with id " << cell_id.id << std::endl;
-	if (parent_id.id == interactive_cell) {
+	if (interactive_cells.find(parent_id.id) != interactive_cells.end()) {
+		interactive_cells.insert(cell_id.id);
 		docthread->on_interactive_output(root);
-		console_child_ids.push_back(cell_id.id);
-		}
-	else if (cell_id.id == interactive_cell || std::find(console_child_ids.begin(), console_child_ids.end(), parent_id.id) != console_child_ids.end()) {
+	}
+	else if (interactive_cells.find(cell_id.id) != interactive_cells.end()) {
 		docthread->on_interactive_output(root);
 		}
 	else if (msg_type.find("csl_") == 0) {
 		root["header"]["from_server"] = true;
 		docthread->on_interactive_output(root);
 		}
+	else if (msg_type == "progress_update") {
+		std::string msg = content.value<std::string>("msg", "Idle");
+		int n = content.value<int>("n", 0);
+		int total = content.value<int>("total", 0);
+		int pulse = content.value<bool>("pulse", false);
+		docthread->set_progress(msg, n, total);
+	}
 	else if(msg_type=="completed") {
 		// std::cerr << "received completion of " << content["original"] << " -> " << content["completed"] << std::endl;
 
@@ -439,11 +446,11 @@ void ComputeThread::on_message(websocketpp::connection_hdl hdl, message_ptr msg)
 	gui->process_data();
 	}
 
-void ComputeThread::execute_interactive(const std::string& code)
+void ComputeThread::execute_interactive(uint64_t id, const std::string& code)
 	{
 	assert(gui_thread_id == std::this_thread::get_id());
 
-	if (!connection_is_open || interactive_cell == 0)
+	if (!connection_is_open)
 		return;
 
 	if (code.substr(0, 7) == "reset()")
@@ -452,7 +459,7 @@ void ComputeThread::execute_interactive(const std::string& code)
 	nlohmann::json req, header, content;
 
 	header["msg_type"]    = "execute_request";
-	header["cell_id"]     = interactive_cell;
+	header["cell_id"]     = id;
 	header["interactive"] = true;
 	content["code"]       = code.c_str();
 
@@ -463,11 +470,7 @@ void ComputeThread::execute_interactive(const std::string& code)
 	std::ostringstream oss;
 	oss << req << std::endl;
 	wsclient.send(our_connection_hdl, oss.str(), websocketpp::frame::opcode::text);
-	}
-
-void ComputeThread::register_interactive_cell(uint64_t id)
-	{
-	interactive_cell = id;
+	interactive_cells.insert(id);
 	}
 
 void ComputeThread::execute_cell(DTree::iterator it)
@@ -621,18 +624,18 @@ bool ComputeThread::complete(DTree::iterator it, int pos, int alternative)
 	size_t lst=todo.find_last_of("\n(){}[]:\t ");
 	if(lst!=std::string::npos)
 		todo=todo.substr(lst+1);
-	// std::cerr << "to complete strip: " << todo << std::endl;	
+	// std::cerr << "to complete strip: " << todo << std::endl;
 
 	if(todo.size()==0)
 		return false;
-	
+
 	req["string"]=todo;
 	req["position"]=pos;
 	req["alternative"]=alternative;
-	
+
 	std::ostringstream str;
 	str << req << std::endl;
-	
+
 	// std::cerr << str.str() << std::endl;
 
 	server_pid=0;
