@@ -62,26 +62,28 @@ namespace cadabra {
 		if (ex1->size() == 0) return ex2;
 		if (ex2->size() == 0) return ex1;
 
-		bool comma1 = (*ex1->begin()->name == "\\comma");
-		bool comma2 = (*ex2->begin()->name == "\\comma");
+		// Using '+' for list addition is very illogical; it should add
+		// componentwise.
+//		bool comma1 = (*ex1->begin()->name == "\\comma");
+//		bool comma2 = (*ex2->begin()->name == "\\comma");
 
-		if (comma1 || comma2) {
-			if (comma1) {
-				auto ret = std::make_shared<Ex>(*ex1);
-				auto loc = ret->append_child(ret->begin(), top2);
-				if (comma2)
-					ret->flatten_and_erase(loc);
-				return ret;
-				}
-			else {
-				auto ret = std::make_shared<Ex>(top2);
-				auto loc = ret->prepend_child(ret->begin(), ex1->begin());
-				if (comma1)
-					ret->flatten_and_erase(loc);
-				return ret;
-				}
-			}
-		else {
+//		if (comma1 || comma2) {
+//			if (comma1) {
+//				auto ret = std::make_shared<Ex>(*ex1);
+//				auto loc = ret->append_child(ret->begin(), top2);
+//				if (comma2)
+//					ret->flatten_and_erase(loc);
+//				return ret;
+//				}
+//			else {
+//				auto ret = std::make_shared<Ex>(top2);
+//				auto loc = ret->prepend_child(ret->begin(), ex1->begin());
+//				if (comma1)
+//					ret->flatten_and_erase(loc);
+//				return ret;
+//				}
+//			}
+//		else {
 			auto ret = std::make_shared<Ex>(*ex1);
 			if (*ret->begin()->name != "\\sum")
 				ret->wrap(ret->begin(), str_node("\\sum"));
@@ -90,7 +92,7 @@ namespace cadabra {
 			auto it = ret->begin();
 			cleanup_dispatch(*get_kernel_from_scope(), *ret, it);
 			return ret;
-			}
+//			}
 		}
 
 	Ex_ptr Ex_mul(const Ex_ptr ex1, const Ex_ptr ex2)
@@ -283,6 +285,17 @@ namespace cadabra {
 		return str.str();
 		}
 
+	Ex_comparator::match_t ExNode_compare(const ExNode& lhs, const ExNode& rhs, const std::string& use_props, bool ignore_parent_rel)
+	{
+		Ex_comparator comp(get_kernel_from_scope()->properties);
+		Ex_comparator::useprops_t up = Ex_comparator::useprops_t::always;
+		if (use_props == "not_at_top")
+			up = Ex_comparator::useprops_t::not_at_top;
+		else if (use_props == "never")
+			up = Ex_comparator::useprops_t::never;
+		return comp.equal_subtree(lhs.it, rhs.it, up, ignore_parent_rel);
+	}
+
 	std::string print_tree(Ex *ex)
 		{
 		std::ostringstream str;
@@ -405,7 +418,7 @@ namespace cadabra {
 		{
 		return ex->to_integer(); // this will throw an exception if the object is not integer
 		}
-	
+
 	std::string Ex_head(Ex_ptr ex)
 		{
 		if (ex->begin() == ex->end())
@@ -549,6 +562,16 @@ namespace cadabra {
 		.export_values()
 		;
 
+		pybind11::enum_<Ex_comparator::match_t>(m, "match_t")
+			.value("node_match", Ex_comparator::match_t::node_match)
+			.value("subtree_match", Ex_comparator::match_t::subtree_match)
+			.value("match_index_less", Ex_comparator::match_t::match_index_less)
+			.value("match_index_greater", Ex_comparator::match_t::match_index_greater)
+			.value("no_match_indexpos_less", Ex_comparator::match_t::no_match_indexpos_less)
+			.value("no_match_indexpos_greater", Ex_comparator::match_t::no_match_indexpos_greater)
+			.value("no_match_less", Ex_comparator::match_t::no_match_less)
+			.value("no_match_greater", Ex_comparator::match_t::no_match_greater);
+
 		pybind11::class_<Ex, Ex_ptr >(m, "Ex")
 		.def(py::init(&Ex_from_string), py::arg("input_form"), py::arg("make_ref") = true, py::arg("kernel") = nullptr)
 		.def(py::init(&Ex_from_int), py::arg("num"), py::arg("make_ref") = true)
@@ -568,23 +591,25 @@ namespace cadabra {
 		.def("__setitem__", &Ex_setitem)
 		.def("__setitem__", &Ex_setitem_iterator)
 		.def("__len__", &Ex_len)
-		.def("__int__", &Ex_int_cast)			
+		.def("__int__", &Ex_int_cast)
 		.def("head", &Ex_head)
 		.def("mult", &Ex_get_mult)
 		.def("__iter__", &Ex_iter)
 		.def("top", &Ex_top)
 		.def("matches", &Ex_matches)
-		.def("matches", &Ex_matches_Ex)			
+		.def("matches", &Ex_matches_Ex)
 		.def("state", &Ex::state)
 		.def("reset", &Ex::reset_state)
+		.def("copy", [](const Ex& ex) { return std::make_shared<Ex>(ex); })
 		.def("changed", &Ex::changed_state)
 		.def("cleanup", &Ex_cleanup)
+		.def("__hash__", [](const Ex& ex) { return ex.calc_hash(ex.begin()); })
 		.def("__add__", static_cast<Ex_ptr(*)(const Ex_ptr, const ExNode)>(&Ex_add), py::is_operator{})
 		.def("__add__", static_cast<Ex_ptr(*)(const Ex_ptr, const Ex_ptr)>(&Ex_add), py::is_operator{})
 		.def("__sub__", static_cast<Ex_ptr(*)(const Ex_ptr, const ExNode)>(&Ex_sub), py::is_operator{})
 		.def("__sub__", static_cast<Ex_ptr(*)(const Ex_ptr, const Ex_ptr)>(&Ex_sub), py::is_operator{})
 		.def("__mul__", static_cast<Ex_ptr(*)(const Ex_ptr, const Ex_ptr)>(&Ex_mul), py::is_operator{})
-		.def(py::pickle(
+			.def(py::pickle(
 			[](const Ex_ptr& ex) { return py::make_tuple(Ex_as_input(ex)); },
 			[](py::tuple t) { return Ex_from_string(t[0].cast<std::string>(), true, get_kernel_from_scope()); }
 		));
@@ -612,13 +637,15 @@ namespace cadabra {
 			.def("append_child", &ExNode::append_child_it, "Append the node pointed to by the given ExNode as a child of the node pointed to by the ExNode.")
 			.def("erase", &ExNode::erase, "Erase the node pointed to by the ExNode.")
 			.def("ex", &ExNode::get_ex, "Obtain a copy of the node pointed to by the ExNode.")
+			.def("compare", &ExNode_compare,
+				py::arg("other"), py::arg("use_props") = "always", py::arg("ignore_parent_rel") = false)
 			.def_property("name", &ExNode::get_name, &ExNode::set_name, "Set the name property of the node pointed to by the ExNode.")
 			.def_property("parent_rel", &ExNode::get_parent_rel, &ExNode::set_parent_rel)
 			.def_property("multiplier", &ExNode::get_multiplier, &ExNode::set_multiplier)
 			.def("__add__", [](ExNode a, Ex_ptr b) {
 								 return a.add_ex(b);
 								 }, pybind11::is_operator{});
-		
+
 		pybind11::class_<sympy::SympyBridge, std::shared_ptr<sympy::SympyBridge> >(m, "SympyBridge")
 		.def(py::init(&SympyBridge_init))
 		.def("to_sympy", &sympy::SympyBridge::export_ex)
