@@ -39,16 +39,25 @@ NotebookWindow::NotebookWindow(Cadabra *c, bool ro)
 	, console(sigc::mem_fun(this, &NotebookWindow::interactive_execute))
 	, current_canvas(0)
 	, kernel_spinner_status(false)
+	, progress_frac(0)
+	, status_line(-1)
+	, status_col(-1)
 	, title_prefix("Cadabra: ")
 	, modified(false)
 	, read_only(ro)
 	, crash_window_hidden(true)
+	, last_configure_width(0)
 	, follow_cell(doc.end())
+	, tex_running(false), tex_need_width(0)
 	, last_find_location(doc.end(), std::string::npos)
 	, is_configured(false)
+
 	{
 	// Connect the dispatcher.
 	dispatcher.connect(sigc::mem_fun(*this, &NotebookWindow::process_todo_queue));
+	dispatch_refresh.connect(sigc::mem_fun(*this, &NotebookWindow::refresh_after_tex_engine_run));
+	dispatch_tex_error.connect(sigc::mem_fun(*this, &NotebookWindow::handle_thread_tex_error));
+	dispatch_update_status.connect(sigc::mem_fun(*this, &NotebookWindow::update_status));
 
 	// Set the window icon.
 	set_icon_name("cadabra2-gtk");
@@ -154,17 +163,25 @@ NotebookWindow::NotebookWindow(Cadabra *c, bool ro)
 	actiongroup->add(Gtk::Action::create("MenuConsoleVisibility", "Console"));
 
 	auto cv_action_hide = Gtk::RadioAction::create(group_cv, "ConsoleHide", "Hide");
-	cv_action_hide->property_value() = 0;
-	actiongroup->add(cv_action_hide, sigc::bind(sigc::mem_fun(*this, &NotebookWindow::on_prefs_set_cv), 0));
+	cv_action_hide->property_value() = (int)Console::Position::Hidden;
+	actiongroup->add(cv_action_hide, sigc::bind(sigc::mem_fun(*this,
+		&NotebookWindow::on_prefs_set_cv), Console::Position::Hidden));
 	cv_action_hide->set_active();
 
-	auto cv_action_show = Gtk::RadioAction::create(group_cv, "ConsoleDock", "Dock");
-	cv_action_show->property_value() = 1;
-	actiongroup->add(cv_action_show, sigc::bind(sigc::mem_fun(*this, &NotebookWindow::on_prefs_set_cv), 1));
+	auto cv_action_dockh = Gtk::RadioAction::create(group_cv, "ConsoleDockH", "Dock (Horizontal)");
+	cv_action_dockh->property_value() = (int)Console::Position::DockedH;
+	actiongroup->add(cv_action_dockh, sigc::bind(sigc::mem_fun(*this,
+		&NotebookWindow::on_prefs_set_cv), Console::Position::DockedH));
+
+	auto cv_action_dockv = Gtk::RadioAction::create(group_cv, "ConsoleDockV", "Dock (Vertical)");
+	cv_action_dockv->property_value() = (int)Console::Position::DockedV;
+	actiongroup->add(cv_action_dockv, sigc::bind(sigc::mem_fun(*this,
+		&NotebookWindow::on_prefs_set_cv), Console::Position::DockedV));
 
 	auto cv_action_float = Gtk::RadioAction::create(group_cv, "ConsoleFloat", "Float");
-	cv_action_float->property_value() = 2;
-	actiongroup->add(cv_action_float, sigc::bind(sigc::mem_fun(*this, &NotebookWindow::on_prefs_set_cv), 2));
+	cv_action_float->property_value() = (int)Console::Position::Floating;
+	actiongroup->add(cv_action_float, sigc::bind(sigc::mem_fun(*this,
+		&NotebookWindow::on_prefs_set_cv), Console::Position::Floating));
 
 
 	Gtk::RadioAction::Group group_font_size;
@@ -264,7 +281,7 @@ NotebookWindow::NotebookWindow(Cadabra *c, bool ro)
 	   "      <separator/>"
 	   "      <menuitem action='Save'/>"
 	   "      <menuitem action='SaveAs'/>"
-	   "      <menuitem action='ExportAsJupyter'/>"		
+	   "      <menuitem action='ExportAsJupyter'/>"
 	   "      <menuitem action='ExportHtml'/>"
 	   "      <menuitem action='ExportHtmlSegment'/>"
 	   "      <menuitem action='ExportLaTeX'/>"
@@ -287,11 +304,11 @@ NotebookWindow::NotebookWindow(Cadabra *c, bool ro)
 		   "      <menuitem action='EditSplit' />"
 		   "      <separator/>"
 		   "      <menuitem action='EditFind' />"
-		   "      <menuitem action='EditFindNext' />"			
+		   "      <menuitem action='EditFindNext' />"
 		   "      <separator/>"
 		   "      <menuitem action='EditMakeCellTeX' />"
 		   "      <menuitem action='EditMakeCellPython' />"
-		   "      <menuitem action='EditIgnoreCellOnImport' />"			
+		   "      <menuitem action='EditIgnoreCellOnImport' />"
 		   "    </menu>"
 		   "    <menu action='MenuView'>"
 		   "      <menuitem action='ViewSplit' />"
@@ -310,7 +327,8 @@ NotebookWindow::NotebookWindow(Cadabra *c, bool ro)
 		   "      </menu>"
 		   "      <menu action='MenuConsoleVisibility'>"
 		   "        <menuitem action='ConsoleHide'/>"
-		   "        <menuitem action='ConsoleDock'/>"
+		   "        <menuitem action='ConsoleDockH'/>"
+			"        <menuitem action='ConsoleDockV'/>"
 		   "        <menuitem action='ConsoleFloat'/>"
 		   "      </menu>"
 		   "      <menuitem action='ViewUseDefaultSettings'/>"
@@ -378,7 +396,8 @@ NotebookWindow::NotebookWindow(Cadabra *c, bool ro)
 	topbox.pack_start(*toolbar, Gtk::PACK_SHRINK);
 	topbox.pack_start(supermainbox, true, true);
 	topbox.pack_start(statusbarbox, false, false);
-	supermainbox.pack_start(mainbox, true, true);
+	supermainbox.pack_start(dragbox, true, true);
+	dragbox.add1(mainbox);
 	mainbox.pack_start(searchbar, false, false);
 	searchbar.add(search_hbox);
 //	searchbar.set_halign(Gtk::ALIGN_START);
@@ -394,13 +413,16 @@ NotebookWindow::NotebookWindow(Cadabra *c, bool ro)
 	status_label.set_size_request(200,-1);
 	status_label.set_justify(Gtk::JUSTIFY_LEFT);
 	kernel_label.set_justify(Gtk::JUSTIFY_LEFT);
+	kernel_label.set_text("Server: not connected");
 	statusbarbox.pack_start(status_label);
 	statusbarbox.pack_start(kernel_label);
 	statusbarbox.pack_start(kernel_spinner);
+//	auto context = kernel_spinner.get_style_context();
+//	context->add_class("spinner");
 	statusbarbox.pack_start(progressbar);
 	statusbarbox.set_name("statusbar");
 	progressbar.set_size_request(200,-1);
-	progressbar.set_text("idle");
+	progressbar.set_text("Idle");
 	progressbar.set_show_text(true);
 
 	searchentry.signal_search_changed().connect(sigc::mem_fun(*this, &NotebookWindow::on_search_text_changed));
@@ -462,7 +484,9 @@ void NotebookWindow::load_css()
 	data += "textview.error { background: transparent; -GtkWidget-cursor-aspect-ratio: 0.2; color: @theme_fg_color; }\n";
 	data += "#ImageView { transition-property: padding, background-color; transition-duration: 1s; }\n";
 	data += "#CodeInput { font-family: monospace; }\n";
-	data += "#Console   { padding: 5px; }\n";
+	data += "#Console   { padding: 2px; }\n";
+	data += "spinner { background: none; opacity: 1; -gtk-icon-source: -gtk-icontheme(\"process-working-symbolic\"); }\n";
+	data += "spinner:checked { opacity: 1; animation: spin 1s linear infinite; }\n";
 
 	// Some of the css properties defined in gtk-cadabra.css are overridden, and so are included
 	// here to force them to be used
@@ -524,67 +548,89 @@ bool NotebookWindow::on_delete_event(GdkEventAny* event)
 		return true;
 	}
 
-void NotebookWindow::on_prefs_set_cv(int vis)
+void NotebookWindow::on_prefs_set_cv(Console::Position pos)
 	{
 	// Unparent from whatever we're currently a child of
-	if (console.get_parent() == nullptr) {
-		// Hidden, do nothing
+	console.hide();
+	if (console.get_parent() == &dragbox) {
+		dragbox.remove(console);
 		}
-	else if (console.get_parent() == &mainbox) {
-		// Docked
-		mainbox.remove(console);
-		}
-	else {
-		// Floating
-		console_win.get_vbox()->remove(console);
+	else if (console.get_parent() == console_win.get_content_area()) {
+		console_win.get_content_area()->remove(console);
 		console_win.hide();
 		}
 
-	// Add to the required container
-	if (vis == 1) {
-		console.set_height(200);
-		mainbox.pack_end(console, false, 5);
+	// Reparent onto something new
+	if (pos == Console::Position::DockedH) {
+		dragbox.set_orientation(Gtk::Orientation::ORIENTATION_HORIZONTAL);
+		dragbox.set_position(dragbox.get_allocation().get_width() / 2);
+		dragbox.add2(console);
 		console.show();
 		}
-	else if (vis == 2) {
+	else if (pos == Console::Position::DockedV) {
+		dragbox.set_orientation(Gtk::Orientation::ORIENTATION_VERTICAL);
+		dragbox.set_position(dragbox.get_allocation().get_height() / 2);
+		dragbox.add2(console);
+		console.set_size_request(-1, 200);
+		console.show();
+		}
+	else if (pos == Console::Position::Floating) {
 		console_win.set_transient_for(*this);
-		console_win.set_resizable(false);
-		console_win.set_size_request(900, 300);
+		console_win.set_resizable(true);
+		console_win.set_default_size(900, 300);
 		console_win.set_title("Interactive Console");
-		console_win.get_vbox()->add(console);
+		console_win.get_content_area()->add(console);
 		console_win.signal_response().connect([this](int) {
 			actiongroup->get_action("ConsoleHide")->activate();
 			});
-		console.set_height(300);
 		console_win.show_all();
 		}
 	}
 
 bool NotebookWindow::on_configure_event(GdkEventConfigure *cfg)
 	{
-	//	std::cerr << "cadabra-client: on_configure_event " << cfg->width << " x " << cfg->height << std::endl;
+	// std::cerr << "cadabra-client: on_configure_event " << cfg->width << " x " << cfg->height << std::endl;
 	is_configured=true;
-	if(cfg->width != last_configure_width)
-		engine.set_geometry(cfg->width-2*30);
 
 	bool ret=Gtk::Window::on_configure_event(cfg);
 
 	if(cfg->width != last_configure_width) {
+		std::lock_guard<std::recursive_mutex> guard(tex_need_width_mutex);
+		if(tex_running) {
+			tex_need_width = cfg->width;
+			}
+		else {
+			tex_need_width = cfg->width;
+			// std::cerr << "Attempting to run" << std::endl;
+			tex_run_async();
+			// std::cerr << "OK, running" << std::endl;
+			}
 		last_configure_width = cfg->width;
-		try {
-			engine.invalidate_all();
-			engine.convert_all();
-			for(unsigned int i=0; i<canvasses.size(); ++i)
-				canvasses[i]->refresh_all();
-			}
-		catch(TeXEngine::TeXException& ex) {
-			on_tex_error(ex.what(), doc.end());
-			}
 		}
 
 	return ret;
 	}
 
+void NotebookWindow::refresh_after_tex_engine_run()
+	{
+	for(unsigned int i=0; i<canvasses.size(); ++i)
+		canvasses[i]->refresh_all();
+	}
+
+void NotebookWindow::handle_thread_tex_error()
+	{
+	on_tex_error(tex_error_string, doc.end());
+	std::cerr << "clicked away" << std::endl;
+	tex_error_string="";
+	}
+
+bool NotebookWindow::on_unhandled_error(const std::exception& err)
+{
+	Gtk::MessageDialog md(*this, err.what(), false, Gtk::MessageType::MESSAGE_ERROR, Gtk::ButtonsType::BUTTONS_OK, true);
+	md.run();
+	return true;
+}
+
 void NotebookWindow::set_title_prefix(const std::string& pf)
 	{
 	title_prefix=pf;
@@ -606,6 +652,17 @@ void NotebookWindow::update_title()
 		}
 	}
 
+void NotebookWindow::set_statusbar_message(const std::string& message, int line, int col)
+	{
+		{
+		std::lock_guard<std::mutex> guard(status_mutex);
+		status_string = message;
+		status_line = line;
+		status_col = col;
+		}
+	dispatch_update_status.emit();
+	}
+
 void NotebookWindow::set_stop_sensitive(bool s)
 	{
 	Gtk::Widget *stop = uimanager->get_widget("/ToolBar/EvaluateStop");
@@ -624,14 +681,17 @@ void NotebookWindow::on_connect()
 	{
 	std::lock_guard<std::mutex> guard(status_mutex);
 	kernel_string = "connected";
+	progress_string = "Idle";
 	dispatcher.emit();
+	dispatch_update_status.emit();
 	console.initialize();
 	// prefs.python_path might end in a backslash which will raise an EOF syntax error, so we add a
 	// semicolon to the end of it and then remove the (empty) last element of the resulting list
 	if (!trim(prefs.python_path).empty())
-		console.send_input("sys.path = r'''" + prefs.python_path + ";'''.split(';')[:-1] + sys.path");
-	if (!name.empty())
-		console.send_input("sys.path.insert(0, r'''" + name.substr(0, name.find_last_of("\\/")) + "''')");
+		console.send_input("sys.path = r'''" + escape_backslashes(prefs.python_path) + ";'''.split(';')[:-1] + sys.path");
+	if (!name.empty()) {
+		console.send_input("sys.path.insert(0, '''" + escape_backslashes(name.substr(0, name.find_last_of("\\/"))) + "''')");
+		}
 	}
 
 void NotebookWindow::on_disconnect(const std::string& reason)
@@ -655,6 +715,53 @@ void NotebookWindow::on_kernel_runstatus(bool running)
 	dispatcher.emit();
 	}
 
+void NotebookWindow::tex_run_async()
+	{
+	if(tex_error_string!="")
+		return;
+
+	// Run TeX on a separate thread; it'll take a while and
+	// we don't want to block the UI thread.
+	// FIXME: join properly when exiting the program, otherwise
+	// we get a 'terminate called without active exception' message.
+	auto tex_code = [this]() {
+							 try {
+								 for(;;) {
+									 {  std::lock_guard<std::recursive_mutex> guard(tex_need_width_mutex);
+										 engine.invalidate_all();
+										 engine.set_geometry(tex_need_width-2*30);
+										 }
+
+									 engine.convert_all();
+									 dispatch_refresh.emit();
+
+									 {  std::lock_guard<std::recursive_mutex> guard(tex_need_width_mutex);
+										 if(tex_need_width-2*30 == engine.get_geometry()) {
+											 tex_running=false;
+											 break;
+											 }
+										 }
+									 }
+								 }
+							 catch(TeXEngine::TeXException& ex) {
+								 // Display the error in the main thread.
+								 std::lock_guard<std::recursive_mutex> guard(tex_need_width_mutex);
+								 tex_error_string=ex.what();
+								 tex_running=false;
+								 dispatch_tex_error.emit();
+								 }
+						 };
+
+	std::lock_guard<std::recursive_mutex> guard(tex_need_width_mutex);
+	tex_running=true;
+
+	// Join any thread which may still exist.
+	if(tex_thread)
+		tex_thread->join();
+
+	tex_thread = std::make_unique<std::thread>( tex_code );
+	}
+
 void NotebookWindow::process_todo_queue()
 	{
 	static bool running=false;
@@ -666,8 +773,7 @@ void NotebookWindow::process_todo_queue()
 	// Update the status/kernel messages into the corresponding widgets.
 		{
 		std::lock_guard<std::mutex> guard(status_mutex);
-		kernel_label.set_text(kernel_string);
-		status_label.set_text(status_string);
+		kernel_label.set_text("Server: " + kernel_string);
 
 		if(kernel_spinner_status) {
 			kernel_spinner.show();
@@ -834,6 +940,8 @@ void NotebookWindow::add_cell(const DTree& tr, DTree::iterator it, bool visible)
 					global_buffer=ci->buffer;
 					}
 				else ci = new CodeInput(it, global_buffer,scale/display_scale,prefs);
+				using namespace std::placeholders;
+				ci->relay_cursor_pos(std::bind(&NotebookWindow::set_statusbar_message, this, "", _1, _2));
 				if(read_only)
 					ci->edit.set_editable(false);
 				ci->get_style_context()->add_provider(css_provider, GTK_STYLE_PROVIDER_PRIORITY_USER);
@@ -939,6 +1047,43 @@ void NotebookWindow::on_interactive_output(const nlohmann::json& msg)
 	console.signal_message(msg);
 	}
 
+void NotebookWindow::set_progress(const std::string& msg, int cur_step, int total_steps)
+	{
+		{
+		std::lock_guard<std::mutex> guard(status_mutex);
+		if (total_steps == 0) {
+			progress_string = msg;
+			progress_frac = 0.0;
+			}
+		else {
+			progress_frac = (double)cur_step / total_steps;
+			progress_string = msg + " (" + std::to_string(cur_step) + "/" + std::to_string(total_steps) + ")";
+			}
+		}
+	dispatch_update_status.emit();
+	}
+
+void NotebookWindow::update_status()
+	{
+	// This should only be called from dispatch_update_status!
+
+	// Update status and progress, kernel status is taken card of in process_action_queue
+	std::lock_guard<std::mutex> guard(status_mutex);
+	progressbar.set_text(progress_string);
+	progressbar.set_fraction(progress_frac);
+
+	if (status_line < 0 || status_col < 0) {
+		status_label.set_text(status_string);
+		}
+	else {
+		std::string pos = " Line: " + std::to_string(status_line) + "   Col: " + std::to_string(status_col);
+		if (status_string == "")
+			status_label.set_text(pos);
+		else
+			status_label.set_text(pos + "   |   " + status_string);
+		}
+	}
+
 void NotebookWindow::remove_cell(const DTree& doc, DTree::iterator it)
 	{
 	// Remember: this member should only remove the visual cell; the
@@ -958,7 +1103,7 @@ void NotebookWindow::remove_cell(const DTree& doc, DTree::iterator it)
 		last_find_location.first=doc.end();
 		last_find_location.second=std::string::npos;
 		}
-	
+
 	DTree::iterator parent = DTree::parent(it);
 	assert(doc.is_valid(parent));
 
@@ -988,7 +1133,7 @@ void NotebookWindow::remove_cell(const DTree& doc, DTree::iterator it)
 		// that it may actually be an inbox or outbox.
 		// std::cerr << "Removing " << actual.imagebox << std::endl;
 		parentbox->remove(*actual.imagebox);
-		
+
 		// The above does not delete the Gtk widget, despite having been
 		// wrapped in manage at construction. So we have to delete it
 		// ourselves. Fortunately the container does not try to delete
@@ -1032,7 +1177,7 @@ void NotebookWindow::update_cell(const DTree&, DTree::iterator it)
 	{
 	// We just do a redraw for now, but this may require more work later.
 	disable_stacks=true;
-	
+
 	for(unsigned int i=0; i<canvasses.size(); ++i) {
 		VisualCell& vc = canvasses[i]->visualcells[&(*it)];
 		if(it->cell_type==DataCell::CellType::python || it->cell_type==DataCell::CellType::latex) {
@@ -1075,7 +1220,7 @@ void NotebookWindow::select_range(const DTree&, DTree::iterator it, int start, i
 		std::cerr << "Warning: select_range called on cell which is not python or latex." << std::endl;
 		return;
 		}
-	
+
 	if(canvasses[current_canvas]->visualcells.find(&(*it))==canvasses[current_canvas]->visualcells.end()) {
 		std::cerr << "cadabra-client: Cannot find cell to select range." << std::endl;
 		return;
@@ -1114,7 +1259,7 @@ void NotebookWindow::scroll_cell_into_view(DTree::iterator cell)
 	{
 //	std::cerr << "-----" << std::endl;
 //	std::cerr << "cell content to show: " << cell->textbuf << std::endl;
-	
+
 	if(current_canvas>=(int)canvasses.size()) return;
 
 	if(canvasses[current_canvas]->visualcells.find(&(*cell))==canvasses[current_canvas]->visualcells.end()) return;
@@ -1135,13 +1280,13 @@ void NotebookWindow::scroll_cell_into_view(DTree::iterator cell)
 		al=focusbox.imagebox->get_allocation();
 	else
 		return;
-	
+
 	Glib::RefPtr<Gtk::Adjustment> va=canvasses[current_canvas]->scroll.get_vadjustment();
 
 	double upper_visible=va->get_value();
 	double page_size    =va->get_page_size();
 	double lower_visible=va->get_value()+page_size;
-	
+
 
 	// When we get called, the busybox has its allocation done and size set. However,
 	// the edit box below still has its old position (but its correct height). So we
@@ -1294,7 +1439,9 @@ bool NotebookWindow::cell_got_focus(DTree::iterator it, int canvas_number)
 
 void NotebookWindow::interactive_execute()
 	{
-	compute->execute_interactive(console.grab_input());
+	uint64_t id = 0;
+	std::string input = console.grab_input(id);
+	compute->execute_interactive(id, input);
 	}
 
 bool NotebookWindow::cell_complete_request(DTree::iterator it, int pos, int canvas_number)
@@ -1506,7 +1653,7 @@ void NotebookWindow::on_file_save_as()
 			else {
 				modified=false;
 				update_title();
-			console.send_input("sys.path.remove(r'''" + old_name.substr(0, old_name.find_last_of("\\/")) + "'''); sys.path.insert(0, r'''" + name.substr(0, name.find_last_of("\\/")) + "''')");
+				console.send_input("sys.path.insert(0, r'''" + escape_backslashes(name.substr(0, name.find_last_of("\\/"))) + "''')");
 				}
 			break;
 			}
@@ -1533,7 +1680,7 @@ void NotebookWindow::on_file_save_as_jupyter()
 			auto ipynb = cnb2ipynb(json);
 			std::ofstream file(ipynb_name);
 			file << ipynb.dump(3) << std::endl;
-			
+
 //			if(res.size()>0) {
 //				Gtk::MessageDialog md("Error saving Jupyter notebook "+name);
 //				md.set_transient_for(*this);
@@ -1795,7 +1942,7 @@ void NotebookWindow::on_search_text_changed()
 		}
 	else {
 		}
-	
+
 	auto res = find_string(start.first, start.second, text, search_case_insensitive.get_active());
 	last_find_location=res;
 	last_find_string=text;
@@ -1924,6 +2071,14 @@ void NotebookWindow::on_kernel_restart()
 	// FIXME: add warnings
 
 	compute->restart_kernel();
+
+		{
+		std::lock_guard<std::mutex> guard(status_mutex);
+		progress_string = "Idle";
+		progress_frac = 0.0;
+		}
+
+	dispatch_update_status.emit();
 	}
 
 void NotebookWindow::on_help() const
@@ -1961,7 +2116,6 @@ void NotebookWindow::on_help() const
 void NotebookWindow::set_compute_thread(ComputeThread* cthread)
 	{
 	DocumentThread::set_compute_thread(cthread);
-	compute->register_interactive_cell(console.get_id());
 	}
 
 void NotebookWindow::on_help_about()
@@ -1986,7 +2140,7 @@ void NotebookWindow::on_help_about()
 	std::vector<Glib::ustring> special;
 	special.push_back("José M. Martín-García (for the xPerm canonicalisation code)");
 	special.push_back("Dominic Price (for the conversion to pybind and most of the Windows port)");
-	special.push_back("Connor Behan (for various improvements related to index-free algorithms)");	
+	special.push_back("Connor Behan (for various improvements related to index-free algorithms)");
 	special.push_back("James Allen (for writing much of the factoring code)");
 	special.push_back("Software Sustainability Institute");
 	special.push_back("Institute of Advanced Study (for a Christopherson/Knott fellowship)");
@@ -2074,7 +2228,7 @@ void NotebookWindow::on_text_scaling_factor_changed(const std::string& key)
 		std::cout << "cadabra-client: total scaling-factor = " << scale << std::endl;
 		engine.set_scale(scale, screen->get_monitor_scale_factor(0));
 		engine.invalidate_all();
-		engine.convert_all();
+		tex_run_async();
 
 		auto it=canvasses.begin();
 		while(it!=canvasses.end()) {
@@ -2173,7 +2327,7 @@ void NotebookWindow::compare_git_latest()
 		auto commit = run_git_command("log --pretty=format:%h -n 1");
 		compare_git(trim(commit));
 		}
-	catch (const std::runtime_error& ex) {
+	catch (const std::exception& ex) {
 		Gtk::MessageDialog error_dialog(ex.what(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
 		error_dialog.set_transient_for(*this);
 		error_dialog.set_title("Git error");
@@ -2200,7 +2354,6 @@ class GitChooseModelColumns : public Gtk::TreeModel::ColumnRecord {
 
 void NotebookWindow::compare_git_choose()
 	{
-#ifndef _MSC_VER
 	try {
 		std::string commit_hash;
 		std::string max_entries = "15";
@@ -2246,18 +2399,12 @@ void NotebookWindow::compare_git_choose()
 
 		compare_git(trim(commit_hash));
 		}
-	catch (const std::runtime_error& ex) {
+	catch (const std::exception& ex) {
 		Gtk::MessageDialog error_dialog(ex.what(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
 		error_dialog.set_transient_for(*this);
 		error_dialog.set_title("Git error");
 		error_dialog.run();
 		}
-#else
-	Gtk::MessageDialog not_supported_dialog("Due to a bug in the Windows version of Gtkmm this feature isn't currently supported. Sorry for the inconvenience!");
-	not_supported_dialog.set_transient_for(*this);
-	not_supported_dialog.set_title("Feature not supported");
-	not_supported_dialog.run();
-#endif
 	}
 
 void NotebookWindow::compare_git_specific()
@@ -2279,7 +2426,7 @@ void NotebookWindow::compare_git_specific()
 
 		compare_git(trim(commit_hash));
 		}
-	catch (const std::runtime_error& ex) {
+	catch (const std::exception& ex) {
 		Gtk::MessageDialog error_dialog(ex.what(), false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
 		error_dialog.set_transient_for(*this);
 		error_dialog.set_title("Git error");
@@ -2310,7 +2457,7 @@ void NotebookWindow::on_prefs_font_size(int num)
 		// on_configure_event signal; that will come after us and then we will
 		// have to run all again.
 		engine.invalidate_all();
-		engine.convert_all();
+		tex_run_async();
 
 		for(auto& canvas: canvasses) {
 			for(auto& visualcell: canvas->visualcells) {
@@ -2491,7 +2638,7 @@ void NotebookWindow::on_tools_clear_cache()
 		? "Cache cleared!"
 		: "Failed to remove cached library files. You can manually remove them by deleting the cadabra_packages folder in your config directory"
 	);
-	
+
 	prog.run();
 }
 
